import java.util.NoSuchElementException;

public interface BoundedSortedMap<K extends Comparable<K>, V> {
    // Overview: Tipo modificabile che rappresenta una funzione i cui elementi del dominio
    //           sono piu' piccoli di un valore prefissato
    // Typical Element: <max, f> con f: K -> V tale che forall k. f(k) definito -> k <= max
    // Notazione: Nel seguito, indicheremo dom(f) = { k | f(k) definito }

    /* restituisce true se la mappa ha un valore associato alla chiave key */
    public boolean containsKey(Object key) throws NullPointerException;
    /*
     REQUIRES: key != null
     THROWS: se key = null lancia NullPointerException
     EFFECTS: restituisce true se key appartiene a dom(f), false altrimenti
     */

    /* elimina la chiave key; restituisce il valore precedentemente associato alla chiave,
       se presente, null altrimenti */
    public V remove(Object key) throws NullPointerException;
    /*
     REQUIRES: key != null
     THROWS: se key = null lancia NullPointerException
     MODIFIES: f
     EFFECTS: rimuove key da dom(f); se key apparteneva a dom(f) restituisce f(key),
              null altrimenti
     */

    /* associa il valore value alla chiave key; restituisce il valore precedentemente
       associato alla chiave, se presente, null altrimenti */
    public V put(K key, V value) throws KeyOutOfBoundException;
    /*
     REQUIRES: key != null && value != null
     THROWS: se key = null o value = null lancia NullPointerException
             se key > max lancia una KeyOutOfBoundException (non presente in Java, checked)
     MODIFIES: f
     EFFECTS: aggiunge key al dominio di f con valore value; se key apparteneva a dom(f)
              restituisce f(key), null altrimenti
     */
    
    /* restituisce la piu' piccola chiave che ha associato un valore */
    public K firstKey( ) throws NoSuchElementException;
    /*
     REQUIRES: dom(f) non vuoto
     THROWS: se dom(f) vuoto lancia NoSuchElementException
     EFFECTS: restituisce la piu' piccola chiave in dom(f)
     */

    /* restituisce una mappa ristretta alla chiavi piu' grandi o uguali a fromKey */
    public BoundedSortedMap<K, V> tailMap(K fromKey);
    /*
     REQUIRES: fromKey != null
     THROWS: se fromKey = null lancia NullPointerException
     EFFECTS: restituisce una funzione g che coincide con f ma il cui dominio e'
              dom(f) intersecato { k | k >= fromKey }
     */
}

class KeyOutOfBoundException extends Exception {
    public KeyOutOfBoundException() {
        super();
    }
    
    public KeyOutOfBoundException(String s) {
        super(s);
    }
}
