import java.util.NoSuchElementException;

public class MyBoundedSortedMap<K extends Comparable<K>, V> implements BoundedSortedMap<K,V> {
    // AF: <maxKey, f> con f(k) = values[i] se exists i. 0 <= i < numKeys && k = keys[i],
    //                     f(k) = indefinito altrimenti
    // IR: maxKey != null && keys != null && values != null &&
    //     (forall i. 0 <= i < numKeys => keys[i] != null && keys[i] <= maxKey &&
    //                                   values[i] != null) &&
    //     (forall i,j. 0 <= i < j < numKeys => keys[i] != keys[j])

    private K maxKey;
    private int numKeys;
    private K[] keys;
    private V[] values;
    
    public MyBoundedSortedMap(K max, K[] k, V[] v) throws KeyOutOfBoundException, NullPointerException {
        if ((max == null) || (k == null) || (v == null))
            throw new NullPointerException();
        for (int i = 0; i < k.length; i++) {
            if ((k[i] == null) || (v[i] == null)) throw new NullPointerException();
            if (k[i].compareTo(max) > 0) throw new KeyOutOfBoundException();
        }
        
        maxKey = max;
        keys = k;
        values = v;
        numKeys = k.length;
    }

    /* restituisce true se la mappa ha un valore associato alla chiave key */
    public boolean containsKey(Object key) throws NullPointerException {
        if (key == null) throw new NullPointerException();
        
        for (int i = 0; i < numKeys; i++)
            if (keys[i].equals(key)) return true;
        return false;
    }
    /*
     REQUIRES: key != null
     THROWS: se key == null lancia NullPointerException
     EFFECTS: restituisce true se esiste i tale che keys[i] == key, false altrimenti
     */
    
    /* elimina la chiave key; restituisce il valore precedentemente associato alla chiave,
       se presente, null altrimenti */
    public V remove(Object key) throws NullPointerException {
        if (key == null) throw new NullPointerException();
        
        for (int i = 0; i < numKeys; i++)
            if (keys[i].equals(key)) {
                V tmp = values[i];
                numKeys--;
                keys[i] = keys[numKeys];
                values[i] = values[numKeys];
                return tmp;
            }
        
        return null;
    }
    /*
     REQUIRES: key != null
     THROWS: se key = null lancia NullPointerException
     MODIFIES: keys, values, numKeys
     EFFECTS: rimuove key da keys; se key occorreva in keys in posizione i restituisce values[i],
              null altrimenti
     */
    
    /* associa il valore value alla chiave key; restituisce il valore precedentemente
       associato alla chiave, se presente, null altrimenti */
    public V put(K key, V value) throws KeyOutOfBoundException, NullPointerException {
        if ((key == null) || (value == null))
            throw new NullPointerException();
        if (key.compareTo(maxKey) > 0) throw new KeyOutOfBoundException();

        for (int i = 0; i < numKeys; i++)
            if (keys[i].equals(key)) {
                V tmp = values[i];
                values[i] = value;
                return tmp;
            }
        
        if (numKeys == keys.length) dup( );

        keys[numKeys] = key; values[numKeys] = value; numKeys++;
        return null;
    }
    /*
     REQUIRES: key != null && value != null && key <= maxKey
     THROWS: se key = null o value = null lancia NullPointerException
             se key > maxKey lancia una KeyOutOfBoundException (non presente in Java, checked)
     MODIFIES: keys, values, numKeys
     EFFECTS: aggiunge key a keys e value a values; se key occorreva gia' in keys in
              posizione i restituisce values[i], null altrimenti
     */

    private void dup() { } // lasciato al lettore
    
    /* restituisce la piu' piccola chiave che ha associato un valore */
    public K firstKey( ) throws NoSuchElementException {
        if (numKeys == 0) throw new NoSuchElementException();

        K key = keys[0];
        
        for (int i = 1; i < numKeys; i++)
            if (keys[i].compareTo(key) < 0)
                key = keys[i];

        return key;
    }
    /*
     REQUIRES: numKeys > 0
     THROWS: se numKeys = 0 lancia NoSuchElementException
     EFFECTS: restituisce key[j] tale che forall i. 0 <= i < numKeys -> keys[j] <= keys[i]
     */
    
    /* restituisce una mappa ristretta alla chiavi piu' grandi o uguali a fromKey */
    public MyBoundedSortedMap<K,V> tailMap(K fromKey) { return this; } // lasciato al lettore
    /*
     REQUIRES: fromKey != null
     THROWS: se fromKey = null lancia NullPointerException
     EFFECTS: restituisce un oggetto la cui funzione associata coincide con quella
              associata a this ma il cui dominio e' { k | k occorre in keys && k >= fromKey }
     */
}
