public class MyIntervalSortedMap<K extends Comparable<K>, V> extends MyBoundedSortedMap<K,V> {
    // AF: <maxKey, numKeys, keys, values>
    // IR: maxKey != null && keys != null && values != null &&
    //     (forall i. 0 <= i < numKeys => keys[i] != null && minKey <= keys[i] <= maxKey &&
    //                                   values[i] != null) &&
    //     (forall i,j. 0 <= i < j < numKeys => keys[i] != keys[j])

    private K minKey;
    
    public MyIntervalSortedMap(K min, K max, K[] k, V[] v) throws KeyOutOfBoundException, NullPointerException {
        super(max, k, v);

        if (min == null) throw new NullPointerException();

        for (int i = 0; i < k.length; i++)
            if (k[i].compareTo(min) < 0) throw new KeyOutOfBoundException();

        minKey = min;
    }

    /* associa il valore value alla chiave key; restituisce il valore precedentemente
       associato alla chiave, se presente, null altrimenti */
    public V put(K key, V value) throws KeyOutOfBoundException, NullPointerException {
        if (key == null) throw new NullPointerException();
        if (key.compareTo(minKey) < 0) throw new KeyOutOfBoundException();

        return super.put(key, value);
     }
    /*
     REQUIRES: key != null && value != null && minKey <= key <= maxKey
     THROWS: se key = null o value = null lancia NullPointerException
             se key > maxKey lancia una KeyOutOfBoundException (non presente in Java, checked)
             se key < minKey lancia una KeyOutOfBoundException (non presente in Java, checked)
     MODIFIES: keys, values, numKeys
     EFFECTS: aggiunge key a keys e value a values; se key occorreva gia' in keys in
              posizione i restituisce values[i], null altrimenti
     */
}
